@@ -5,12 +5,12 @@
 /// @author  Sascha Krieg
 /// @author  Michael Behrisch
 /// @date    Wed, 12 Dez 2001
-/// @version $Id: MSJunction.h 17262 2014-11-04 20:41:44Z behrisch $
+/// @version $Id: MSJunction.h 19412 2015-11-24 13:04:53Z namdre $
 ///
 // The base class for an intersection
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -49,6 +49,8 @@ class MSLink;
 class MSLane;
 class MSEdge;
 
+typedef std::vector<const MSEdge*> ConstMSEdgeVector;
+
 // ===========================================================================
 // class definitions
 // ===========================================================================
@@ -97,11 +99,11 @@ public:
         return myEmptyLanes;
     }
 
-    inline const std::vector<const MSEdge*>& getIncoming() const {
+    inline const ConstMSEdgeVector& getIncoming() const {
         return myIncoming;
     }
 
-    inline const std::vector<const MSEdge*>& getOutgoing() const {
+    inline const ConstMSEdgeVector& getOutgoing() const {
         return myOutgoing;
     }
 
@@ -118,6 +120,12 @@ public:
         return myType;
     }
 
+    /// @brief erase vehicle from myLinkLeaders
+    void passedJunction(const MSVehicle* vehicle);
+
+    /* @brief @return whether the foe vehicle is a leader for ego
+     * @note vehicles are added to myLinkLeaders when first seen as a foe */
+    bool isLeader(const MSVehicle* ego, const MSVehicle* foe);
 
 protected:
     /// @brief Tye type of this junction
@@ -134,13 +142,18 @@ protected:
 
 
     /// @brief incoming edges
-    std::vector<const MSEdge*> myIncoming;
+    ConstMSEdgeVector myIncoming;
     /// @brief outgoing edges
-    std::vector<const MSEdge*> myOutgoing;
+    ConstMSEdgeVector myOutgoing;
 
     /// @brief definition of the static dictionary type
     typedef std::map<std::string, MSJunction* > DictType;
 
+    /// @brief map from leader vehicle to follower vehicles
+    typedef std::map<const MSVehicle*, std::set<const MSVehicle*> > LeaderMap;
+    LeaderMap myLinkLeaders;
+
+
 private:
     /// @brief Invalidated copy constructor.
     MSJunction(const MSJunction&);
