@@ -6,7 +6,7 @@
 /// @author  Thimor Bohn
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: NIImporter_ArcView.cpp 18752 2015-08-30 19:09:05Z namdre $
+/// @version $Id: NIImporter_ArcView.cpp 19523 2015-12-04 11:00:56Z behrisch $
 ///
 // Importer for networks stored in ArcView-shape format
 /****************************************************************************/
@@ -275,17 +275,16 @@ NIImporter_ArcView::load() {
         if (dir == "B" || dir == "F" || dir == "" || myOptions.getBool("shapefile.all-bidirectional")) {
             if (myEdgeCont.retrieve(id) == 0) {
                 LaneSpreadFunction spread = dir == "B" || dir == "FALSE" ? LANESPREAD_RIGHT : LANESPREAD_CENTER;
-                NBEdge* edge = new NBEdge(id, from, to, type, speed, nolanes, priority, width, NBEdge::UNSPECIFIED_OFFSET, shape, "", spread);
+                NBEdge* edge = new NBEdge(id, from, to, type, speed, nolanes, priority, width, NBEdge::UNSPECIFIED_OFFSET, shape, name, id, spread);
                 myEdgeCont.insert(edge);
                 checkSpread(edge);
             }
         }
         // add negative direction if wanted
         if (dir == "B" || dir == "T" || myOptions.getBool("shapefile.all-bidirectional")) {
-            id = "-" + id;
-            if (myEdgeCont.retrieve(id) == 0) {
+            if (myEdgeCont.retrieve("-" + id) == 0) {
                 LaneSpreadFunction spread = dir == "B" || dir == "FALSE" ? LANESPREAD_RIGHT : LANESPREAD_CENTER;
-                NBEdge* edge = new NBEdge(id, to, from, type, speed, nolanes, priority, width, NBEdge::UNSPECIFIED_OFFSET, shape.reverse(), "", spread);
+                NBEdge* edge = new NBEdge("-" + id, to, from, type, speed, nolanes, priority, width, NBEdge::UNSPECIFIED_OFFSET, shape.reverse(), name, id, spread);
                 myEdgeCont.insert(edge);
                 checkSpread(edge);
             }
