@@ -4,12 +4,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: NIVissimAbstractEdge.cpp 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: NIVissimAbstractEdge.cpp 19210 2015-10-29 10:31:59Z behrisch $
 ///
 // -------------------
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -36,7 +36,6 @@
 #include <utils/common/MsgHandler.h>
 #include <utils/common/ToString.h>
 #include <utils/geom/GeomHelper.h>
-#include <utils/geom/Line.h>
 #include <utils/geom/GeoConvHelper.h>
 #include <netbuild/NBNetBuilder.h>
 #include "NIVissimAbstractEdge.h"
@@ -97,11 +96,9 @@ NIVissimAbstractEdge::getGeomPosition(SUMOReal pos) const {
         return myGeom[-1];
     } else {
         PositionVector g(myGeom);
-        SUMOReal amount = pos - myGeom.length();
-        Position ne = GeomHelper::extrapolate_second(g[-2], g[-1], amount * 2);
-        g.pop_back();
-        g.push_back(ne);
-        return g.positionAtOffset(pos);
+        const SUMOReal amount = pos - myGeom.length();
+        g.extrapolate(amount * 2);
+        return g.positionAtOffset(pos + amount * 2);
     }
 }
 
@@ -129,22 +126,10 @@ NIVissimAbstractEdge::crossesEdge(NIVissimAbstractEdge* c) const {
 
 Position
 NIVissimAbstractEdge::crossesEdgeAtPoint(NIVissimAbstractEdge* c) const {
-    return myGeom.intersectsAtPoint(c->myGeom);
+    return myGeom.intersectionPosition2D(c->myGeom);
 }
 
 
-SUMOReal
-NIVissimAbstractEdge::crossesAtPoint(const Position& p1,
-                                     const Position& p2) const {
-    // !!! not needed
-    Position p = GeomHelper::intersection_position2D(
-                     myGeom.front(), myGeom.back(), p1, p2);
-    return GeomHelper::nearest_offset_on_line_to_point2D(
-               myGeom.front(), myGeom.back(), p);
-}
-
-
-
 std::vector<int>
 NIVissimAbstractEdge::getWithin(const AbstractPoly& p, SUMOReal offset) {
     std::vector<int> ret;
