@@ -4,12 +4,12 @@
 /// @author  Daniel Krajzewicz
 /// @author  Felix Brack
 /// @date    2006
-/// @version $Id: OutputDevice_Network.cpp 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: OutputDevice_Network.cpp 18095 2015-03-17 09:39:00Z behrisch $
 ///
 // An output device for TCP/IP Network connections
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2006-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2006-2015 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -30,6 +30,11 @@
 #include <config.h>
 #endif // #ifdef _MSC_VER
 
+#ifdef WIN32
+#include <windows.h>
+#else
+#include <unistd.h>
+#endif
 #include <vector>
 #include "OutputDevice_Network.h"
 #include "foreign/tcpip/socket.h"
@@ -46,10 +51,20 @@
 OutputDevice_Network::OutputDevice_Network(const std::string& host,
         const int port) {
     mySocket = new tcpip::Socket(host, port);
-    try {
-        mySocket->connect();
-    } catch (tcpip::SocketException& e) {
-        throw IOError(toString(e.what()) + " (host: " + host + ", port: " + toString(port) + ")");
+    for (int wait = 1000; true; wait += 1000) {
+        try {
+            mySocket->connect();
+            break;
+        } catch (tcpip::SocketException& e) {
+            if (wait == 9000) {
+                throw IOError(toString(e.what()) + " (host: " + host + ", port: " + toString(port) + ")");
+            }
+#ifdef WIN32
+            Sleep(wait);
+#else
+            usleep(wait * 1000);
+#endif
+        }
     }
 }
 
